//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* Aqualizer GUI.cpp
 * 
 * The GUI of the Aqualizer PWS EQ by Jelle and Josse VST.
 * Should be used together with PWS EQ.cpp.
 * 
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 * 
 * Contents:
     1. Constructor and destructor
     2. Open and close functions
     3. Functions to make some bands blue or gray
     4. Parameter-related functions
 */

#include "PWS EQ.h"
//#include "Aqualizer GUI.h"  this is done in "PWS EQ.h"

//-----------------------------------------------------------------------------
// 1. Constructor and destructor
//-----------------------------------------------------------------------------
AQGUI::AQGUI (AudioEffect* effect)
: AEffGUIEditor (effect)
{
    eq = (PWSEQ*) effect;
    
    bandSliders = new CVerticalSlider*[kNumBands];
    BWKnobs = new CAnimKnob*[kNumBands];
    freqEdits = new CTextEdit*[kNumBands];
    
    for (int i = 0; i < kNumBands; i++)
    {
        bandSliders[i] = 0;
        BWKnobs[i] = 0;
        freqEdits[i] = 0;
    }
    
    gainSlider = 0;
    BWKnob = 0;
    BWLink = 0;
    numDisplay = 0;
    numButton = 0;

    goButton = 0;
    minFreq = 0;
    maxFreq = 0;
    fMinFreq = 20.0f;
    fMaxFreq = 20000.0f;

    bBGR = new CBitmap (kBGR);
    
    rect.left   = 0;
    rect.top    = 0;
    rect.right  = (short) bBGR->getWidth ();
    rect.bottom = (short) bBGR->getHeight ();

    blueClr (204, 238, 255, 216);
    grayClr (0, 0, 0, 0);
}

//-----------------------------------------------------------------------------
AQGUI::~AQGUI ()
{
    if (bBGR)
        bBGR->forget ();
    bBGR = 0;
    
    //if (frame) close ();
    delete[] bandSliders;
    delete[] BWKnobs;
}

//-----------------------------------------------------------------------------
// 2. Open and close functions
//-----------------------------------------------------------------------------
bool AQGUI::open (void* ptr)
{
    //if (ptr != 0)
        AEffGUIEditor::open (ptr);        //always call this!
    
    CBitmap* bBar = new CBitmap (kbBar);
    CBitmap* bHandle = new CBitmap (kbHandle);
    CBitmap* bKnob = new CBitmap (kbKnob);
    CBitmap* bFreq = new CBitmap (kbFreq);

    CBitmap* gBar = new CBitmap (kgBar);
    CBitmap* gHandle = new CBitmap (kgHandle);
    CBitmap* gKnob = new CBitmap (kgKnob);
    CBitmap* gFreq = new CBitmap (kgFreq);

    CBitmap* Link = new CBitmap (kLink);
    CBitmap* GO = new CBitmap (kGo);
    CBitmap* bNumBands = new CBitmap (kNumBandsBitmap);
    
    CRect size (0, 0, bBGR->getWidth(), bBGR->getHeight());
    CFrame* lFrame = new CFrame (size, ptr, this);
    lFrame->setBackground (bBGR);
    
    CPoint offset (0, 0);
    
    NumBandsActive = eq->getNumBandsActive ();
    BWLinked = eq->getBWLinked ();
    
    int i;
    
    for (i = 0; i < NumBandsActive; i++)
    {
        // kBand1 to kBand32 (volume) faders
        size (xBar + i * dx, yBar, xBar + i * dx + bHandle->getWidth(), yBar + bBar->getHeight());
        bandSliders[i] = new CVerticalSlider (size, this, kBand1 + i,
                                              size.top, size.bottom - bHandle->getHeight(),
                                              bHandle, bBar, offset, kBottom);
        (bandSliders[i])->setDefaultValue (0.5f);
        (bandSliders[i])->setValue (effect->getParameter (kBand1 + i));
        lFrame->addView (bandSliders[i]);
        
        // kBW1 to kBW32 bandwidth knobs
        size (xKnob + i * dx, yKnob, xKnob + i * dx + bKnob->getWidth(), yKnob + kHeightOfSub);
        BWKnobs[i] = new CAnimKnob (size, this, kBW1 + i,
                                    kNumKnobSubs, (CCoord) kHeightOfSub,
                                    bKnob, offset);
        (BWKnobs[i])->setDefaultValue (effect->getParameter (kBW));
        (BWKnobs[i])->setValue (effect->getParameter (kBW1 + i));
        lFrame->addView (BWKnobs[i]);

        // kFreq1 to kFreq32 frequency TextEdits
        size (xFreq + i * dx, yFreq, xFreq + i * dx + bFreq->getWidth(), yFreq + bFreq->getHeight());
        freqEdits[i] = new CTextEdit (size, this, kFreq1 + i,
                                      0, bFreq, kCenterText);
        (freqEdits[i])->setFont (kNormalFontVerySmall);
        (freqEdits[i])->setFontColor (blueClr);
        (freqEdits[i])->setTextEditConvert (txtConvertAndCut);
        char txt[256];
        char str[256];
        float val = effect->getParameter (kFreq1 + i) * effect->getSampleRate() / 2.0f;
        int lg = (int)floor(log10(val));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), val);
        txtConvert (txt, str, false);
        (freqEdits[i])->setText (str);
        lFrame->addView (freqEdits[i]);
    }
    for (; i < kNumBands; i++)
    {
        // kBand1 to kBand32 (volume) faders
        size (xBar + i * dx, yBar, xBar + i * dx + bHandle->getWidth(), yBar + bBar->getHeight());
        bandSliders[i] = new CVerticalSlider (size, this, kBand1 + i,
                                              size.top, size.bottom - bHandle->getHeight(),
                                              gHandle, gBar, offset, kBottom);
        (bandSliders[i])->setDefaultValue (0.5f);
        (bandSliders[i])->setValue (effect->getParameter (kBand1 + i));
        lFrame->addView (bandSliders[i]);
        
        // kBW1 to kBW32 bandwidth knobs
        size (xKnob + i * dx, yKnob, xKnob + i * dx + gKnob->getWidth(), yKnob + kHeightOfSub);
        BWKnobs[i] = new CAnimKnob (size, this, kBW1 + i,
                                    kNumKnobSubs, (CCoord) kHeightOfSub,
                                    gKnob, offset);
        (BWKnobs[i])->setDefaultValue (effect->getParameter (kBW));
        (BWKnobs[i])->setValue (effect->getParameter (kBW1 + i));
        lFrame->addView (BWKnobs[i]);

        // kFreq1 to kFreq32 frequency TextEdits
        size (xFreq + i * dx, yFreq, xFreq + i * dx + gFreq->getWidth(), yFreq + gFreq->getHeight());
        freqEdits[i] = new CTextEdit (size, this, kFreq1 + i,
                                      0, gFreq, kCenterText);
        (freqEdits[i])->setFont (kNormalFontVerySmall);
        (freqEdits[i])->setFontColor (grayClr);
        (freqEdits[i])->setTextEditConvert (txtConvertAndCut);
        char txt[256];
        char str[256];
        float val = effect->getParameter (kFreq1 + i) * effect->getSampleRate() / 2.0f;
        int lg = (int)floor(log10(val));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), val);
        txtConvert (txt, str, false);
        (freqEdits[i])->setText (str);
        lFrame->addView (freqEdits[i]);
    }
    
    // gain fader
    size (xGainBar, yGainBar, xGainBar + bBar->getWidth(), yGainBar + bBar->getHeight());
    gainSlider = new CVerticalSlider (size, this, kGain,
                                      size.top, size.bottom - bHandle->getHeight(),
                                      bHandle, bBar, offset, kBottom);
    gainSlider->setDefaultValue (0.5f);
    gainSlider->setValue (effect->getParameter (kGain));
    lFrame->addView (gainSlider);

    // overall BW
    size (xLastKnob, yLastKnob, xLastKnob + bKnob->getWidth(), yLastKnob + kHeightOfSub);
    BWKnob = new CAnimKnob (size, this, kBW,
                            kNumKnobSubs, (CCoord) kHeightOfSub,
                            bKnob, offset);
    BWKnob->setValue (effect->getParameter (kBW));
    lFrame->addView (BWKnob);

    // link BW
    size (xLinkBW, yLinkBW, xLinkBW + Link->getWidth(), yLinkBW + Link->getHeight() / 2);
    BWLink = new COnOffButton (size, this, kLinkBW,
                               Link);
    BWLink->setValue (effect->getParameter (kLinkBW));
    lFrame->addView (BWLink);

    // num Bands button
    size (xNumButton, yNumButton, xNumButton + bNumBands->getWidth () / 3, yNumButton + bNumBands->getHeight ());
    numButton = new CIncButton (size, this, kNumBandsActive,
                                bNumBands, 1, 32, eq->getNumBandsActive ());
    lFrame->addView (numButton);

    // num Bands display
    size (xNumDisplay, yNumDisplay, xNumDisplay + bFreq->getWidth (), yNumDisplay + bFreq->getHeight ());
    numDisplay = new CParamDisplay (size, bFreq, kCenterText);
    numDisplay->setFont (kNormalFontSmall);
    numDisplay->setFontColor (grayClr);
    numDisplay->setStringConvert (strConvert);
    numDisplay->setValue (((float) eq->getNumBandsActive ()) / ((float) kNumBands));
    lFrame->addView (numDisplay);

    // go button
    size (xGo, yGo, xGo + GO->getWidth (), yGo + GO->getHeight () / 2);
    goButton = new CKickButton (size, this, kNumParams,
                                GO, offset);
    goButton->setValue (0.0f);
    lFrame->addView (goButton);

    // minFreq
    {
        char txt[256];
        char str[256];
        int lg = (int)floor(log10(fMinFreq));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), fMinFreq);
        txtConvert (txt, str, false);
        size (xMinFreq, yMinFreq, xMinFreq + bFreq->getWidth (), yMinFreq + bFreq->getHeight ());
        minFreq = new CTextEdit (size, this, kMinFreq,
                                 str, bFreq, kCenterText);
        minFreq->setFont (kNormalFontVerySmall);
        minFreq->setFontColor (blueClr);
        minFreq->setTextEditConvert (txtConvertAndCut);
        lFrame->addView (minFreq);
    }

    // maxFreq
    {
        char txt[256];
        char str[256];
        int lg = (int)floor(log10(fMaxFreq));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), fMaxFreq);
        txtConvert (txt, str, false);
        size (xMaxFreq, yMaxFreq, xMaxFreq + bFreq->getWidth (), yMaxFreq + bFreq->getHeight ());
        maxFreq = new CTextEdit (size, this, kMaxFreq,
                                 str, bFreq, kCenterText);
        maxFreq->setFont (kNormalFontVerySmall);
        maxFreq->setFontColor (blueClr);
        maxFreq->setTextEditConvert (txtConvertAndCut);
        lFrame->addView (maxFreq);
    }
    
    bBar->forget ();
    bHandle->forget ();
    bKnob->forget ();
    bFreq->forget ();

    gBar->forget ();
    gHandle->forget ();
    gKnob->forget ();
    gFreq->forget ();
    
    Link->forget ();
    GO->forget ();
    bNumBands->forget ();
    
    setColors ();
    frame = lFrame;
    return true;
}

//-----------------------------------------------------------------------------
void AQGUI::close ()
{
    delete frame;
    frame = 0;
}

//-----------------------------------------------------------------------------
// 3. Functions to make some bands blue or gray
//-----------------------------------------------------------------------------
bool AQGUI::setBlue (VstInt32 num)
{
    CBitmap* bBar = new CBitmap (kbBar);
    CBitmap* bHandle = new CBitmap (kbHandle);
    CBitmap* bKnob = new CBitmap (kbKnob);
    CBitmap* gKnob = new CBitmap (kgKnob);
    
    CBitmap* bFreq = new CBitmap (kbFreq);
    
    (bandSliders[num])->setBackground (bBar);
    (bandSliders[num])->setHandle (bHandle);

    (freqEdits[num])->setFontColor (blueClr);
    (freqEdits[num])->setBackground (bFreq);

    if (!BWLinked)
    {
        (BWKnobs[num])->setBackground (bKnob); //blue knob
    }
    else
    {
        (BWKnobs[num])->setBackground (gKnob); //gray knob
    }

    bBar->forget ();
    bHandle->forget ();
    bKnob->forget ();
    gKnob->forget ();

    bFreq->forget ();
    return true;
}

//-----------------------------------------------------------------------------
bool AQGUI::setGray (VstInt32 num)
{
    CBitmap* gBar = new CBitmap (kgBar);
    CBitmap* gHandle = new CBitmap (kgHandle);
    CBitmap* gKnob = new CBitmap (kgKnob);
    
    CBitmap* gFreq = new CBitmap (kgFreq);

    (bandSliders[num])->setBackground (gBar);
    (bandSliders[num])->setHandle (gHandle);

    (freqEdits[num])->setFontColor (grayClr);
    (freqEdits[num])->setBackground (gFreq);
    
    (BWKnobs[num])->setBackground (gKnob);

    gBar->forget ();
    gHandle->forget ();
    gKnob->forget ();

    gFreq->forget ();
    return true;
}

//-----------------------------------------------------------------------------
bool AQGUI::setColors ()
{
    NumBandsActive = eq->getNumBandsActive ();
    BWLinked = eq->getBWLinked ();

    CBitmap* bKnob = new CBitmap (kbKnob);
    CBitmap* gKnob = new CBitmap (kgKnob);

    if (BWLinked)
    {
        BWKnob->setBackground (bKnob); //blue knob
    }
    else
    {
        BWKnob->setBackground (gKnob); //gray knob
    }

    bKnob->forget ();
    gKnob->forget ();
    
    int i;
    for (i = 0; i < NumBandsActive; i++)
        setBlue (i);
    for (; i < kNumBands; i++)
        setGray (i);
    return true;
}

//-----------------------------------------------------------------------------
// 4. Parameter-related functions
//-----------------------------------------------------------------------------
void AQGUI::setParameter (VstInt32 index, float value)
{
    if (frame == 0) return;
    
    if (index - kBand1 < kNumBands)
    {
        (bandSliders[index - kBand1])->setValue (effect->getParameter (index));
        return;
    }
    if (index - kFreq1 < kNumBands)
    {
        char txt[256];
        char str[256];
        float val = effect->getParameter (index) * effect->getSampleRate() / 2.0f;
        int lg = (int)floor(log10(val));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), val);
        txtConvert (txt, str, false);
        (freqEdits[index - kFreq1])->setText (str);
        return;
    }
    if (index - kBW1 < kNumBands)
    {
        (BWKnobs[index - kBW1])->setValue (effect->getParameter (index));
        return;
    }
    if (index == kNumBandsActive)
    {
        setColors ();
        numButton->setCurr ((long) eq->getNumBandsActive ()); //hiernaar kijken
        numDisplay->setValue (((float) eq->getNumBandsActive ()) / ((float) kNumBands));
        return;
    }
    if (index == kLinkBW)
    {
        BWLink->setValue (effect->getParameter (index));
        setColors ();
        return;
    }
    if (index == kBW)
    {
        BWKnob->setValue (effect->getParameter (index));
        for (int i = 0; i < kNumBands; i++)
            (BWKnobs[i])->setDefaultValue (effect->getParameter (index));
        return;
    }
    if (index == kGain)
    {
        gainSlider->setValue (effect->getParameter (index));
        return;
    }
	if (index == kMinFreq)
	{
		fMinFreq = effect->getParameter (index);
		char txt[256];
        char str[256];
        float val = fMinFreq * effect->getSampleRate() / 2.0f;
        int lg = (int)floor(log10(val));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), val);
        txtConvert (txt, str, false);
        minFreq->setText (str);
		return;
	}
	if (index == kMaxFreq)
	{
		fMaxFreq = effect->getParameter (index);
		char txt[256];
        char str[256];
        float val = fMaxFreq * effect->getSampleRate() / 2.0f;
        int lg = (int)floor(log10(val));
        sprintf (txt, "%.*f", 4 - lg - (lg == 4 ? 0 : 1), val);
        txtConvert (txt, str, false);
        maxFreq->setText (str);
		return;
	}
}

//-----------------------------------------------------------------------------
void AQGUI::valueChanged (CDrawContext* context, CControl* control)
{
    long tag = control->getTag ();
    if (tag < 0 || tag >= kNumParams + 1) return;
            //we're using one extra parameter: goButton
    
	eq->setGUIChanges (true);
    effect->setParameterAutomated (tag, control->getValue ());
    if (tag >= kFreq1 && tag <= kFreq32)
    {
        char txt[256];
        char str[256];
        float val;
        (freqEdits[tag - kFreq1])->getText (txt);
        txtConvert (txt, str, false);
        (freqEdits[tag - kFreq1])->setText (str);
        sscanf(str, "%f", &val);
        effect->setParameterAutomated (tag, val / effect->getSampleRate() * 2.0f);
    }
    if (tag == kNumBandsActive)
    {
        setColors ();
        numDisplay->setValue (((float) numButton->getCurr ()) / ((float) kNumBands));
        numDisplay->setDirty ();
    }
    if (tag == kBW)
    {
        for (int i = 0; i < kNumBands; i++)
            (BWKnobs[i])->setDefaultValue (effect->getParameter (tag));
    }
    if (tag == kLinkBW) setColors ();
    if (tag == kMinFreq)    //minFreq changed
    {
        char txt[256];
        char str[256];
        float val;
        minFreq->getText (txt);
        txtConvert (txt, str, false);
        minFreq->setText (str);
        sscanf(str, "%f", &val);
        fMinFreq = val;
        effect->setParameterAutomated (tag, val / effect->getSampleRate() * 2.0f);
    }
    if (tag == kMaxFreq)    //maxFreq changed
    {
        char txt[256];
        char str[256];
        float val;
        maxFreq->getText (txt);
        txtConvert (txt, str, false);
        maxFreq->setText (str);
        sscanf(str, "%f", &val);
        fMaxFreq = val;
        effect->setParameterAutomated (tag, val / effect->getSampleRate() * 2.0f);
    }
    if (tag == kNumParams)        //GO button pressed
    {
        eq->calcMinMaxFreqs ();
    }
	eq->setGUIChanges (false);
    control->setDirty ();
}

//-----------------------------------------------------------------------------
void txtConvert (char* input, char* string, bool cut)
{
    int numCommas = 0;
    int j = 0;
    for (int i = 0; i < (int) strlen(input); i++)
    {
        if (input[i] == ',') input[i] = '.';
        if (isdigit(input[i]) || (input[i] == '.' && numCommas == 0))
            string[j++] = input[i];
        if (input[i] == '.' && (!cut || j < 5)) numCommas++;
    }
    if (cut) string[min(5, j)] = '\0';   //maximum length: 5
    else string[j] = '\0';
    while (numCommas != 0 && string[strlen(string)-1] == '0')
        string[strlen(string)-1] = '\0';
    if (string[strlen(string)-1] == '.') string[strlen(string)-1] = '\0';
}

//-----------------------------------------------------------------------------
void txtConvertAndCut (char* input, char* string)
{
    txtConvert (input, string, true);
}

//-----------------------------------------------------------------------------
void strConvert (float value, char* text)
{
    sprintf(text, "%.0f", value * kNumBands);
}
