//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* Aqualizer GUI.h
 * 
 * The GUI of the Aqualizer PWS EQ by Jelle and Josse VST.
 * Should be used together with PWS EQ.cpp.
 * 
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 * 
 * Contents:
     1. Constructor and destructor
     2. Open and close functions
     3. Functions to make some bands blue or gray
     4. Parameter-related functions
 */

#ifndef __AQ_GUI__
#define __AQ_GUI__

#include "vstgui.h"
#include "vstcontrols.h"
#include "vstIncButton.h"

class PWSEQ;

//-----------------------------------------------------------------------------
void txtConvert (char* input, char* string, bool cut = false);
void txtConvertAndCut (char* input, char* string);
void strConvert (float value, char* text);

//-----------------------------------------------------------------------------
enum
{
    //------------------------------------------------
    //bitmap ID's
    //------------------------------------------------
    kBGR = 128,

    kbBar,
    kbHandle,
    kbKnob,
    kbFreq,

    kgBar,
    kgHandle,
    kgKnob,
    kgFreq,
    
    kLink,
    kGo,
    kNumBandsBitmap,

    //------------------------------------------------
    //co?rdinates
    //------------------------------------------------
    dx = 30,                //delta x
    
    // Bars
    yBar = 52,
    xBar = 10,
    
    yGainBar = yBar,
    xGainBar = 994,
    
    // Knobs
    yKnob = 247,
    xKnob = 7,

    yLastKnob = yKnob,
    xLastKnob = 991,

    kNumKnobSubs = 128,
    kHeightOfSub = 30,
    
    // TextEdits
    yFreq = 227,
    xFreq = 5,

    yMinFreq = 293,
    xMinFreq = 495,

    yMaxFreq = 308,
    xMaxFreq = 495,

    // display
    yNumDisplay = yFreq,
    xNumDisplay = 989,

    // Buttons
    yLinkBW = 250,
    xLinkBW = 967,

    yGo = 297,
    xGo = 535,

    yNumButton = 225,
    xNumButton = 972
};

//-----------------------------------------------------------------------------
class AQGUI : public AEffGUIEditor, public CControlListener
{
    public:
    // 1. Constructor and destructor
    AQGUI (AudioEffect* effect);
    ~AQGUI ();
    
    // 2. Open and close functions
    virtual bool open (void* prt);
    virtual void close ();
    
    // 3. Functions to make some bands blue or gray
    virtual bool setBlue (VstInt32 num);
    virtual bool setGray (VstInt32 num);

    virtual bool setColors ();

    // 4. Parameter-related functions
    virtual void setParameter (VstInt32 index, float value);
    virtual void valueChanged (CDrawContext* context, CControl* control);
    
    private:
    CVerticalSlider** bandSliders;
    CVerticalSlider*  gainSlider;
    
    CAnimKnob** BWKnobs;
    CAnimKnob*  BWKnob;

    COnOffButton* BWLink;
    
    CBitmap* bBGR;

    VstInt32 NumBandsActive;
    bool BWLinked;

    CTextEdit** freqEdits;
    CParamDisplay* numDisplay;
    CIncButton* numButton;

    CColor blueClr;
    CColor grayClr;

    PWSEQ* eq;

    CKickButton* goButton;
    CTextEdit* minFreq;
    CTextEdit* maxFreq;

    float fMinFreq, fMaxFreq;        //the minFreq and maxFreq frequencies
};

#endif
