//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* PWS EQ.h
 * 
 * A VST equalizer.
 * It's a 32-band parametric EQ, that can be used as a graphic EQ as well.
 *
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 * 
 * PWSEQ class contents:
     1. Constructor and destructor
     2. canDo and product info
     3. Input and output properties
     4. Settings for audio processing
     5. Parameters
     6. Programs
     7. Midi
     8. Main process
 */

#ifndef __PWS_EQ__
#define __PWS_EQ__

#include <stdio.h>
#include "Aqualizer GUI.h"
#include "peakinghandler.h"
#include "audioeffectx.h"
#include "aeffect.h"

enum
{
    //global settings
    kNumPrograms = 7,         //we've got seven programs
    
    kNumInputs = 2,              //stereo input
    kNumOutputs = 2,          //stereo output
    kNumMidiInputs = 0,       //All midi channels
    kNumMidiOutputs = 0,      //All midi channels
    
    kNumBands = 32,           //maximum of 32 bands; this should be changed in struct programs as well!
    
    //params
    kBand1 = 0,
    kBand2,
    kBand3,
    kBand4,
    kBand5,
    kBand6,
    kBand7,
    kBand8,
    kBand9,
    kBand10,
    kBand11,
    kBand12,
    kBand13,
    kBand14,
    kBand15,
    kBand16,
    kBand17,
    kBand18,
    kBand19,
    kBand20,
    kBand21,
    kBand22,
    kBand23,
    kBand24,
    kBand25,
    kBand26,
    kBand27,
    kBand28,
    kBand29,
    kBand30,
    kBand31,
    kBand32,
    kFreq1,
    kFreq2,
    kFreq3,
    kFreq4,
    kFreq5,
    kFreq6,
    kFreq7,
    kFreq8,
    kFreq9,
    kFreq10,
    kFreq11,
    kFreq12,
    kFreq13,
    kFreq14,
    kFreq15,
    kFreq16,
    kFreq17,
    kFreq18,
    kFreq19,
    kFreq20,
    kFreq21,
    kFreq22,
    kFreq23,
    kFreq24,
    kFreq25,
    kFreq26,
    kFreq27,
    kFreq28,
    kFreq29,
    kFreq30,
    kFreq31,
    kFreq32,
    kBW1,
    kBW2,
    kBW3,
    kBW4,
    kBW5,
    kBW6,
    kBW7,
    kBW8,
    kBW9,
    kBW10,
    kBW11,
    kBW12,
    kBW13,
    kBW14,
    kBW15,
    kBW16,
    kBW17,
    kBW18,
    kBW19,
    kBW20,
    kBW21,
    kBW22,
    kBW23,
    kBW24,
    kBW25,
    kBW26,
    kBW27,
    kBW28,
    kBW29,
    kBW30,
    kBW31,
    kBW32,
    kNumBandsActive,
    kLinkBW,
    kBW,
    kGain,
	kMinFreq,
	kMaxFreq,
    kNumParams
};

class programs
{
    public:
	programs()
	{
		for (int i = 0; i < 32; i++)
			bandGain[i] = 0;
		mainGain = 0;
		freqMin = 20;
		freqMax = 20000;
	}
	float bandGain[32];		//dB
    float freqs[32];		//Hz
    float BWs[32];			//
    float BW;				//
    int BandsActive;		//
    bool BWlink;			//
	float mainGain;			//dB
    char name[24];			//
	float freqMin;			//Hz
	float freqMax;			//Hz
};


class PWSEQ : public AudioEffectX
{
    public:
    // 1. Constructor and destructor
    PWSEQ (audioMasterCallback audioMaster);
    ~PWSEQ ();
    
    // 2. canDo and product info
    virtual VstInt32 canDo (char* text);
    virtual bool getEffectName(char* name);
    virtual bool getVendorString (char* text);
    virtual bool getProductString (char* text);
    virtual VstInt32 getVendorVersion ();
    
    // 3. Input and output properties
    virtual bool getInputProperties (VstInt32 index, VstPinProperties* properties);
    virtual bool getOutputProperties (VstInt32 index, VstPinProperties* properties);
    virtual bool setBypass (bool OnOff);
    
    // 4. Settings for audio processing
    virtual void setSampleRate (float sampleRate);
    virtual void setBlockSize (VstInt32 blockSize);
    
    // 5. Parameters
    virtual void setParameter (VstInt32 index, float value);
    virtual float getParameter (VstInt32 index);
    virtual void getParameterLabel (VstInt32 index, char* label);
    virtual void getParameterDisplay (VstInt32 index, char* text);
    virtual void getParameterName (VstInt32 index, char* text);

    VstInt32 getNumBandsActive ();
    bool getBWLinked ();

    void calcMinMaxFreqs ();

	void setGUIChanges (bool ch);
    
    // 6. Programs
    void initProgs ();
    
    virtual void setProgram (VstInt32 program);
    virtual void setProgramName (char* name);
    virtual VstInt32 getProgram ();
    virtual void getProgramName (char* name);
    virtual bool getProgramNameIndexed (VstInt32 category, VstInt32 index, char* text);
    
    // 7. Midi
    virtual VstInt32 getNumMidiInputChannels ();
    virtual VstInt32 getNumMidiOutputChannels ();
    
    virtual VstInt32 processEvents (VstEvents* events);
    
    void checkEvents (int num);
    
    // 8. Main process
    virtual void processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames);
    virtual void processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames);
    
    
    private:
    peakinghandler* EQ;    //the actual equalizer
    
    float* f;              //EQ frequencies
    float* b;              //EQ bandwidths
    float* p;              //EQ peaks
    float bw;              //overall BW
    
    float* params;         //all the current parameters
    
    programs progs[7];     //the programs
    VstInt32 curProg;      //current program number
    
    float fs;              //sample frequency
    VstInt32 bs;           //block size
    
    bool ByPass;           //true: Bypass
    
    bool BWlinked;
    VstInt32 numBandsActive;
    float gain;
    
    VstEvents* ev;

    bool EditorLoaded;

	float dbRange;         //deciBell range

	bool settingProgram;

	float minFreq;
	float maxFreq;

	bool GUIChanges;
};

#endif
