//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

/* peaking.h
 * 
 * Creates a biquad peaking filter (2 poles, 2 zeros).
 * You can use peakinghandler.cpp to handle multiple peaking filters in series.
 * That way it creates an equalizer.
 * 
 * Made by Jelle van Mourik and Josse van Dobben de Bruyn
 */

#include <math.h>
#define min(a, b)  (((a) < (b)) ? (a) : (b))
#define max(a, b)  (((a) > (b)) ? (a) : (b))

class peaking
{
    public:
    peaking();
    peaking( float freq, float BW, float peak);
    ~peaking();
    
    void setFreq(float freq);
    float getFreq();
    
    void setBW(float BW);
    float getBW();
    
    void setPeak(float peak);
    float getPeak();
    
    void turnOff();
    void turnOn();
    
    float peakingfilterL(float in);
    float peakingfilterR(float in);
    
    private:
    float H(float z);     //calculate frequency response for z = {1, -1}
    // (well, actually for all z on the unit circle, but C++ doesn't support complex numbers)
    void calc();    //calculate co?ffici?nts and gain
    
    float f;        //center frequency
    float b;        //bandwidth
    float p;        //peak
    float pi;
    
    float c;        //cos(f)
    float absz;     //abs(z)
    float absp;     //abs(p)
    
    float xl1, xl2; //x[n-1] and x[n-2] for the left channel
    float yl1, yl2; //y[n-1] and y[n-2]
    
    float xr1, xr2; //x[n-1] and x[n-2] for the right channel
    float yr1, yr2; //y[n-1] and y[n-2]
    
    float a1, a2;
    float b1, b2;
    float gain;
    
    bool fSet;
    bool bSet;
    bool pSet;
    bool canfilter;
};
