//-----------------------------------------------------------------------------
// Copyright  2008, Josse van Dobben de Bruyn and Jelle van Mourik
// All rights reserved.
//
// Basic idea
// This license concerns the use and reproduction of the Aqualizer in its
// compiled version as well as its source code. It was designed for
// non-commercial purposes, and its designers, Josse van Dobben de Bruyn and
// Jelle van Mourik, want to keep it that way, so that as much people as
// possible will be able to use it. You are therefore encouraged to use the
// Aqualizer what it was made for: sound-design. Furthermore, you are allowed
// to read the source-code, learn from it, where possible improve it, and
// publish. However, you are NOT allowed to use the original version of the
// Aqualizer or its code, nor a modified version of it for commercial purposes
// without explicit written permission of both the designers.
// The original version of the Aqualizer can be found at
// http://vst.rjprojects.net/Aqualizer/index.htm.
//
// Redistribution
// Unless explicit written permission of both the designers will state
// otherwise, redistribution and use in source and binary forms, with or
// without modification, are permitted provided that the following conditions
// are met:
//     1) Redistributions of source code must retain the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the disclaimer, at the very top of each source file.
//     2) Redistributions in binary form must reproduce the entire license
//        agreement, consisting of the above copyright notice, the description
//        of the basic idea, this list of conditions, the term of agreement and
//        the following disclaimer in the documentation and/or other materials
//        provided with the distribution. 
//     3) Neither the name of the Aqualizer nor the names of its contributors
//        may be used to endorse or promote products derived from this
//        software.
//     4) The licensee is explicitly forbidden to sell, rent or lease the
//        Aqualizer to a third party, or to use it in any other form for
//        commercial purposes. This also applies to copies or modifications of
//        the Aqualizer.
//
// Term of agreement
// Any violation of this license agreement will end the violators right of use
// of the Aqualizer in its compiled version or its source code, and this
// license and all the rights granted to the licensee herein shall immediately
// cease. If his rights have been terminated, he does not qualify to receive
// new licenses for the same material.
//
// Disclaimer:
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// VstIncButton.cpp
//
// A VST control with an inc-button and a dec-button.
//
// Can be implemeted within a Vst-plugin which uses vstgui.
//
// Made by Jelle van Mourik and Josse van Dobben de Bruyn
//-----------------------------------------------------------------------------

#include "vstIncButton.h"
#include <math.h>

//-----------------------------------------------------------------------------
CIncButton::CIncButton (const CRect &size, CControlListener* listener, long tag,
                        CBitmap* background, long minPos, long maxPos, long startPos)
: CControl (size, listener, tag, background), minPos (minPos), maxPos (maxPos), curr (startPos)
{
    diff = maxPos - minPos + 1;
    cl = 0;
}

//-----------------------------------------------------------------------------
CIncButton::~CIncButton ()
{}

//-----------------------------------------------------------------------------
void CIncButton::draw (CDrawContext* pContext)
{
    if (pBackground)
    {
        CPoint where (0, 0); //start at the beginning of the Bitmap
        
        if (cl == 1)
            where.h += size.right - size.left;
        else if (cl == -1)
            where.h += 2 * (size.right - size.left);

        if (bTransparencyEnabled)
            pBackground->drawTransparent (pContext, size, where);
        else
            pBackground->draw (pContext, size, where);
    }
    setDirty (false);
}

//-----------------------------------------------------------------------------
void CIncButton::mouse (CDrawContext* pContext, CPoint &where, long button)
{
    if (!bMouseEnabled) return;

    if (button = -1) button = pContext->getMouseButtons ();
    if (!(button & kLButton)) return;

    if (listener && button & (kAlt | kShift | kControl | kApple))
    {
        if (listener->controlModifierClicked (pContext, this, button) != 0)
            return;
    }

    if (checkDefaultValue (pContext, button)) return;

    
    float y,h;

    beginEdit ();

    if (pContext->getMouseButtons () == kLButton)
    {
        int lastCl = 0;
        do
        {
            y = (float) (where.v - size.top);
            h = pBackground->getHeight () / 2.0f;
            cl = (y <= h ? 1 : -1);
            if (lastCl != cl)
            {
                if (curr + cl > maxPos || curr + cl < minPos)
                    cl = 0;
                curr += cl;
                if (cl)
                {
                    value = (float) (curr - minPos) / (float) (diff - 1);
                    if (isDirty () && listener)
                        listener->valueChanged (pContext, this);
                }            
            }
            lastCl = cl;

            getMouseLocation (pContext, where);
        
            doIdleStuff ();
        }
        while (pContext->getMouseButtons () == kLButton);
    }
    cl = 0;
    if (listener)
        listener->valueChanged (pContext, this);
    endEdit ();
}

//-----------------------------------------------------------------------------
void CIncButton::setValue (float val)
{
    value = val;
    curr = (int) floor (val * (float) (diff - 1) + 0.5f) + minPos;
}

//-----------------------------------------------------------------------------
void CIncButton::setCurr (long cr)
{
    curr = cr;
    value = (float) (curr - minPos) / (float) (diff - 1);
}

//-----------------------------------------------------------------------------
long CIncButton::getCurr ()
{
    return curr;
}
